#!/bin/bash
# ------------------------------------------------------------------------------
# Copyright 2023 Open Text.
#
# The only warranties for products and services of Open Text and its
# affiliates and licensors (“Open Text”) are as may be set forth in the
# express warranty statements accompanying such products and services.
# Nothing herein should be construed as constituting an additional
# warranty. Open Text shall not be liable for technical or editorial
# errors or omissions contained herein. The information contained herein
# is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains
# confidential information and a valid license is required for possession,
# use or copying. If this work is provided to the U.S. Government,
# consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items
# are licensed to the U.S. Government under vendor's standard commercial
# license.
# ------------------------------------------------------------------------------
#
# script to check all oes-rest config health

UWHITE="\033[4;37m"
URED="\033[4;31m"
UGREEN="\033[4;32m"
GREEN="\033[1;32m"
RED="\033[0;31m"
BLUE="\033[0;94m"
YELLOW="\033[1;33m"
NC="\033[0m"

LOGFILE="/var/opt/novell/log/umc/apps/oes-rest/health.log"
startTime=$(date +%s.%N)
autoFix=false
declare -i serviceCheckIssue=0
declare -i serviceCheckAutoFix=0

VERSION_NUMBER="1.0.0" #script version

logFile_check(){
        if [[ -f $LOGFILE ]]
        then
            echo "#=================================================#" >> $LOGFILE
            echo "" >> $LOGFILE
            printf "#=================================================#\n"
            echo "                [Start OES REST HEALTH]                 " >> $LOGFILE
            echo >> $LOGFILE
            printf "   Script executed on: [$(date +%Y-%m-%d:%H:%M:%S:%Z)]\n"
            echo "   Script executed on: [$(date +%Y-%m-%d:%H:%M:%S:%Z))]  " >> $LOGFILE
            echo >> $LOGFILE
            echo "   Autofix value is set to $autoFix" >> $LOGFILE
            echo >> $LOGFILE
        else
            touch $LOGFILE
            echo "#=================================================#" >> $LOGFILE
            echo >> $LOGFILE
            printf "#=================================================#\n"
            echo "                [Start OES REST HEALTH]                 " >> $LOGFILE
            echo >> $LOGFILE
            printf "   Script executed on: [$(date +%Y-%m-%d:%H:%M:%S:%Z)]\n"
            echo "   Script executed on: [$(date +%Y-%m-%d:%H:%M:%S:%Z))]  " >> $LOGFILE
            echo >> $LOGFILE
            echo "   Autofix value is set to $autoFix" >> $LOGFILE
            echo >> $LOGFILE
        fi
}

service_check(){

	printf "#=================================================#\n\n"
	printf "             [Service Status Check]                \n\n"
	echo "#=================================================#" >> $LOGFILE
	echo >> $LOGFILE
	echo "             [Service Status Check]                " >> $LOGFILE

	array=('apache2' 'microfocus-umc-backend')
	#filter all services to different array on basis of the status of service
	for i in ${array[@]};
	  do
	    result=$(systemctl is-active ${i}.service)
	    if [ "$result" = "active" ]
	    then
	       active_service+=" "${i}
	    elif [ "$result" = "inactive" ]
	    then
	       inactive_service+=" "${i}
	    elif [ "$result" = "failed" ]
	    then
	       failed_service+=" "${i}
	    fi
	  done
	
	#check for active service and dispaly
	for i in ${active_service[@]};
	do
	result=$(systemctl is-active ${i}.service)
        printf "${i}.service is ${UGREEN}${result}${NC}\n"
        echo "${i}.service is ${result}" >> $LOGFILE
	done
	
	#check for failed service and dispaly
	for i in ${failed_service[@]};
	do
	result=$(systemctl is-active ${i}.service)
	printf "${i}.service is ${URED}${result}${NC}\n"
	serviceCheckIssue+=1
	echo "${i}.service is ${result}" >> $LOGFILE
	if [[ "$autoFix" = true ]]
	then
	    printf "${RED}ERROR: ${i}${NC} is in failed state. Starting the service.....\n\n"
	    echo "ERROR: ${i} is in failed state. Starting the service....." >> $LOGFILE
	    $(systemctl restart ${i}.service)
	    printf "Exit value for restarting ${i}.service is : $?\n\n"
	    echo "Exit value for restarting ${i}.service is : $?" >> $LOGFILE
	    if [ $? == 0 ]
	    then
	        serviceCheckAutoFix+=1
	    fi
	else
	    printf "${RED}ERROR: ${i}${NC} is in failed state. Make sure service is up and running by executing ${BLUE}systemctl restart ${i}.service${NC}\n\n"
	    echo "ERROR: ${i} is in failed state. Make sure service is up and running by executing systemctl restart ${i}.service" >> $LOGFIL
	fi
	done
	
	#check for inactive service and dispaly
	for i in ${inactive_service[@]};
	do
	result=$(systemctl is-active ${i}.service)
	printf "${i}.service is ${URED}${result}${NC}\n"
	serviceCheckIssue+=1
	echo "${i}.service is ${result}" >> $LOGFILE
	if [[ "$autoFix" = true ]]
	then
	    printf "${RED}ERROR: ${i}${NC} is in inactive state. Starting the service.....\n\n"
	    echo "ERROR: ${i} is in inactive state. Starting the service....." >> $LOGFILE
	    $(systemctl restart ${i}.service)
	    printf "Exit value for restarting ${i}.service is : $?\n\n"
	    echo "Exit value for restarting ${i}.service is : $?" >> $LOGFILE
	    if [ $? == 0 ]
	    then
	        serviceCheckAutoFix+=1
	    fi
	else
	    printf "${RED}ERROR: ${i}${NC} is in inactive state. Make sure service is up and running by executing ${BLUE}systemctl restart ${i}.service${NC}\n\n"
	    echo "ERROR: ${i} is in inactive state. Make sure service is up and running by executing systemctl restart ${i}.service" >> $LOGFILE
	fi
	done
	
}

apacheModule_check(){
	
	# check the apache module needed for umc
	printf "#=================================================#\n\n"
	echo "#=================================================#" >> $LOGFILE
	echo >> $LOGFILE
	printf "               [Apache Module Check]              \n\n"
	echo "               [Apache Module Check]              " >> $LOGFILE

	header_mod=$(apachectl -M 2> /dev/null | grep proxy_http_module)
	if [ -z "$header_mod" ]
	then
	    serviceCheckIssue+=1
	    if [[ "$autoFix" = true ]]
	    then
	        printf "${RED}ERROR: apache proxy_http module is disabled${NC}\nEnabling the apache proxy_http module.....\n\n"
	        echo "ERROR: apache proxy_http module is disabled. Enabling the apache proxy_http module....." >> $LOGFILE
	        $(a2enmod proxy_http)
	        printf "Exit value for enabling proxy_http module is : $?\n\n"
	        echo "Exit value for enabling proxy_http module is : $?" >> $LOGFILE
	        if [ $? == 0 ]
	        then
	            serviceCheckAutoFix+=1
	        fi
	        $(systemctl restart apache2.service)
	        printf "Exit value for restarting apache2.service is : $?\n\n"
	        echo "Exit value for restarting apache2.service is : $?" >> $LOGFILE
	    else
	        printf "${RED}ERROR: apache proxy_http module is disabled${NC}\nEnable it by running ${BLUE}a2enmod proxy_http${NC} and restart the ${GREEN}apache2${NC} by executing ${BLUE}systemctl restart apache2.service${NC}\n\n"
	        echo "ERROR: apache proxy_http module is disabled. Enable it by running a2enmod proxy_http and restart the apache2 by executing systemctl restart apache2.service" >> $LOGFILE
	    fi
	else
	    printf "apache proxy_http module is ${GREEN}enabled${NC}\n\n"
	    echo "apache proxy_http module is enabled" >> $LOGFILE
	fi

	printf "\n#=================================================#\n"
	echo "#=================================================#" >> $LOGFILE

}

finalSummary() {

	echo "                    [Summary]                       " >> $LOGFILE
	printf "                    [Summary]                       \n"
	if [ $serviceCheckIssue -gt 0 ]
	then
	    echo "      Service Check $serviceCheckIssue Issue Found" >> $LOGFILE
	    printf "      Service Check $serviceCheckIssue Issue Found\n"
	fi

	if [ $serviceCheckAutoFix -gt 0 ]
	then
	    echo "      Service Check $serviceCheckAutoFix Issue AutoFixed" >> $LOGFILE
	    printf "      Service Check $serviceCheckAutoFix Issue AutoFixed\n"
	fi

	if [ $serviceCheckIssue -eq 0 ]
	then
	    echo "                 No Issue Found                   " >> $LOGFILE
	    printf "                 No Issue Found                   \n"
	fi

	echo "#=================================================#" >> $LOGFILE
	printf "#=================================================#\n"

}

indicateEnd() {
    duration=$(awk "BEGIN {print $(date +%s.%N) - $startTime}")
    execution_time=$(printf "%.2f seconds" $duration)
    printf "      Script Execution Time: $execution_time\n\n"
    echo "       Script Execution Time: $execution_time" >> $LOGFILE
    echo "       END on : [$(date +%Y-%m-%d:%H:%M:%S:%Z)]                      " >> $LOGFILE
    printf "      END on : [$(date +%Y-%m-%d:%H:%M:%S:%Z)]                      \n"
    echo "#=================================================#" >> $LOGFILE
    printf "#=================================================#\n"
    exit 0
}

runAllCheck() {

logFile_check
# Check Service Status related to UMC
service_check
# Check apache module is enable or disabled
apacheModule_check
finalSummary
indicateEnd

}

Usage()
{
        echo ""
        echo "oesRestHealth Script Version ${VERSION_NUMBER} checks health of all the services and provide necessary action in case of failure"
        echo ""
        echo "Usage: oesRestHealth.sh [options]"
        echo "          -h|--help                  Prints this summary"
        echo "          -s|--service-check         Check health of dependent services"
        echo "          -a|--all-check             Run all health check for UMC"
        echo ""
        echo "Log Location - ${LOGFILE} "
        echo ""
        echo "Autofix is available for --service-check, and --all-check option."
        echo "Autofix will automatically fix the issue detected."
        echo ""
        echo "Example -> sh oesRestHealth.sh -sautofix OR sh oesRestHealth.sh --service-check=autofix"
        echo "Example -> sh oesRestHealth.sh -aautofix OR sh oesRestHealth.sh --all-check=autofix"
        echo ""
        echo "If you dont want to enable auto fix then run script without passing 'autofix'."
        echo "It will populate the solution if any issue is detected."
        echo ""
        echo "Example -> sh oesRestHealth.sh -s OR sh oesRestHealth.sh --service-check"
        exit 1;
}

if test $# -eq 0
then
        Usage
fi

#Read the options
TEMP=`getopt -o s::a::h --long service-check::,all-check::,help -- "$@"`
eval set -- "$TEMP"
#Extract options and their arguments into variables.
while true;
do
  case "$1" in
          -s|--service-check)
            case "$2" in
                    "autofix") autoFix=true ;logFile_check;service_check;apacheModule_check;finalSummary;indicateEnd; exit;;
                           "") autofix=false ;logFile_check;service_check;apacheModule_check;finalSummary;indicateEnd; exit;;
            esac;;
          -a|--all-check)
            case "$2" in
                    "autofix") autoFix=true ;runAllCheck; exit;;
                           "") autofix=false ;runAllCheck; exit;;
            esac;;
          -h|--help)
             Usage;;
          \?)
             # Invalid option
             echo "Error: Invalid option"
             Usage;;
           *)
             Usage;;
  esac
done
